# Copyright (C) 2007 by Intevation GmbH
# Authors:
# Bernhard Herzog <bh@intevation.de>
#
# This program is free software under the GPL (>=v2)
# Read the file GPL.txt coming with the software for details.


import optparse
from xml.dom.minidom import parse

from dbbackend import DBField, Insertion


class OSAASOption(optparse.Option):

    ATTRS = optparse.Option.ATTRS[:]
    ATTRS.extend(["xml_path"])


class ConfigXMLPathError(Exception):

    pass


def find_xml_path_node(node, path):
    for name in path.split("/"):
        children = [child for child in node.childNodes
                          if child.nodeName == name]
        if not children:
            return None
        if len(children) > 1:
            raise ConfigXMLPathError("More than one element matches %r" % path)
        node = children[0]
    return node

def find_xml_path_text(node, path):
    node = find_xml_path_node(node, path)
    if node is None:
        return None
    node.normalize()
    value_node = node.firstChild
    if value_node.nodeType != value_node.TEXT_NODE:
        raise ConfigXMLPathError("Config file entry %r must only have text"
                                 " content" % path)
    return value_node.data

def read_database_rules(node):
    dbnode = find_xml_path_node(node, "OSAASConfig/DBRules/insert")
    if dbnode is None:
        return []
    value_nodes = [node for node in dbnode.childNodes
                        if node.nodeType == node.ELEMENT_NODE
                           and node.nodeName == "value"]
    def asciiattr(node, attr):
        return node.getAttribute(attr).encode("ascii")
    return [Insertion(asciiattr(dbnode, "table"),
                      [DBField(asciiattr(n, "param"),
                               asciiattr(n, "column"),
                               asciiattr(n, "type"))
                       for n in value_nodes])]


def read_config_file(filename, opts):
    dom = parse(filename)
    for option in opts.option_list:
        if option.dest is not None and option.xml_path:
            value = find_xml_path_text(dom, option.xml_path)
            if value is not None:
                opts.set_file_option(option.dest,
                                     option.check_value(option.xml_path, value))
    opts.set_file_option("db_rules", read_database_rules(dom))
