# Copyright (C) 2007 by Intevation GmbH
# Authors:
# Bernhard Herzog <bh@intevation.de>
#
# This program is free software under the GPL (>=v2)
# Read the file GPL.txt coming with the software for details.

from __future__ import nested_scopes

import logging

from httpserver import HTTPServer
from threadpool import ThreadPool


class ThreadedHTTPServer(HTTPServer):

    def __init__(self, *args, **kw):
        for name, value in [("num_threads", 5)]:
            if kw.has_key(name):
                value = kw[name]
                del kw[name]
            setattr(self, name, value)
        HTTPServer.__init__(self, *args, **kw)

        pool_logger = logging.getLogger(self.error_logger_name + ".workerpool")
        self.thread_pool = ThreadPool(self.num_threads, lambda f: f(),
                                      logger=pool_logger)
        self.thread_pool.start()

    def process_request(self, request, client_address):
        """Put the request into the queue to be handled by a worker thread
        """
        def process_in_worker():
            try:
                self.finish_request(request, client_address)
                self.close_request(request)
            except:
                self.handle_error(request, client_address)
                self.close_request(request)
        self.thread_pool.put(process_in_worker)

    def server_close(self):
        HTTPServer.server_close(self)
        self.thread_pool.stop()
