# Copyright (C) 2007, 2008 by Intevation GmbH
# Authors:
# Bernhard Herzog <bh@intevation.de>
#
# This program is free software under the GPL (>=v2)
# Read the file GPL.txt coming with the software for details.

import logging

from osaas.http.run import ServerProgram
from osaas.server import OSAASServer
from osaas.config import OSAASOption, read_config_file
from osaas.dbbackend import DBBackend, load_db_api

class OSAASProgramMixin:

    optparse_option_class = OSAASOption

    def add_db_options(self, parser):
        for name in ["Type", "Parameters", "User", "Password", "DSN"]:
            option = parser.add_option("--db-" + name.lower())
            option.xml_path = "OSAASConfig/DBConnection/" + name

    def read_config_file(self, opts):
        if opts.config_file is not None:
            read_config_file(opts.config_file, opts)


class OSAASServerProgram(OSAASProgramMixin, ServerProgram):

    def create_option_parser(self, **kw):
        parser = ServerProgram.create_option_parser(self, **kw)
        for name in ["port", "access-log", "error-log", "log-level",
                     "pid-file"]:
            path = "OSAASConfig/" + "".join([part.capitalize()
                                             for part in name.split("-")])
            parser.get_option("--" + name).xml_path = path
        parser.add_option("--userdb-file", xml_path="OSAASConfig/UserDB")
        self.add_db_options(parser)
        return parser

    def instantiate_server(self, server_class, server_address, opts, **kw):
        dbbackend = DBBackend(load_db_api(opts.db_type),
                              (opts.db_parameters,
                               opts.db_user, opts.db_password, opts.db_dsn),
                              opts.db_rules,
                              logging.getLogger("httpserver.error.dbbackend"))
        return server_class(server_address, userdbfile=opts.userdb_file,
                            dbbackend=dbbackend, **kw)



def main():
    program = OSAASServerProgram()
    program.main(server_class=OSAASServer)
