# Copyright (C) 2007 by Intevation GmbH
# Authors:
# Bernhard Herzog <bh@intevation.de>
#
# This program is free software under the GPL (>=v2)
# Read the file GPL.txt coming with the software for details.

"""Simple User database"""

import sha
import base64
import errno

SSHA_PREFIX = "{SSHA}"

def random_salt(length):
    random = open("/dev/urandom")
    try:
        return random.read(length)
    finally:
        random.close()

def encode_ssha(password, salt):
    digester = sha.new(password)
    digester.update(salt)
    return SSHA_PREFIX + base64.b64encode(digester.digest() + salt)

def check_password(password, hashed):
    if hashed.startswith(SSHA_PREFIX):
        salt = base64.b64decode(hashed[len(SSHA_PREFIX):])[20:]
        return encode_ssha(password, salt) == hashed
    else:
        return False

class UserDB(object):

    def __init__(self, users=()):
        self.users = {}
        for username, password in users:
            self.add_user(username, password)

    def read(self, filename):
        try:
            pwfile = open(filename)
        except IOError, err:
            # If the file doesn't exist, treat it as an empty file
            if err.errno == errno.ENOENT:
                return
        try:
            while 1:
                line = pwfile.readline()
                if not line:
                    break

                fields = line.strip().split(":", 1)
                if len(fields) == 2:
                    self.users[fields[0]] = fields[1]
                else:
                    # FIXME: warn about this
                    pass
        finally:
            pwfile.close()

    def write(self, filename):
        pwfile = open(filename, "w")
        for username, pwhash in sorted(self.users.items()):
            pwfile.write("%s:%s\n" % (username, pwhash))
        pwfile.close()

    def add_user(self, username, password):
        self.users[username] = encode_ssha(password, random_salt(8))

    def check_credentials(self, username, password):
        if self.users.has_key(username):
            return check_password(password, self.users[username])
        return False
